﻿using System;
using System.Text;
using System.Windows.Forms;

namespace KosmosSample
{
    /// <summary>
    /// このプログラムは神津精機製のコントローラARIESをRS-232C通信によって制御するサンプルコード(C#)です。
    /// <NR></NR>
    /// このサンプルコードはARIESのRS-232C制御のプログラムを作成する参考に作成されたものです。
    /// 商用・非商用を問わず実用を考慮した設計にはなっていません。
    /// <NR></NR>
    /// 弊社の環境にて動作確認をしておりますが、いかなる環境でも動作することを保証しておりません。
    /// <NR></NR>
    /// 起動をするとシリアルポートコンポーネントを立ち上げ、ポートを開きます。
    /// 次に「IDN」ボタンを押下するとIDNコマンドを送信し、型式を受信します。
    /// このアプリを閉じるときにポートを閉じます。
    /// <NR></NR>   
    /// このサンプルコードの著作権は神津精機株式会社に帰属します。
    /// Copylight　2018　Kohzu Precision Co., Ltd.
    /// written by Masato Okui
    /// </summary>
    public partial class Form1 : Form
    {
        /// <summary>
        /// メインフォームのインスタンスです。
        /// </summary>
        public Form1()
        {
            InitializeComponent();

            //ボーレートを設定します。コントローラのボーレートを書き込んでください。
            //serialPort1.BaudRate = 115200;
            serialPort1.BaudRate = 57600;
            //serialPort1.BaudRate = 38400;

            //ポート番号を設定します。コントローラのCOMポートを指定してください。
            serialPort1.PortName = "COM1";
            //serialPort1.PortName = "COM2";
            //serialPort1.PortName = "COM3";
            //serialPort1.PortName = "COM4";
            //serialPort1.PortName = "COM5";
            //serialPort1.PortName = "COM6";
            //serialPort1.PortName = "COM7";
            //serialPort1.PortName = "COM8";
            //serialPort1.PortName = "COM9";

            //シリアルポートを開きます。
            serialPort1.Open();

            textBox1.Text = String.Format("{0}\t{1}\tポートを開きました。コントローラのボーレートを{2}baudに設定してください。\r\n", 
                DateTime.Now.ToLongTimeString(), serialPort1.PortName, serialPort1.BaudRate);
        }


        /// <summary>
        /// このフォームを閉じる直前に実行されます
        /// </summary>
        /// <param name="sender">送信元のオブジェクトです</param>
        /// <param name="e">イベントデータです</param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            //シリアルポートを閉じます。
            serialPort1.Close();
        }

        //コントローラの型式を格納します。
        private string ControllerType = "Unknown";

        //送信コマンドを格納します。
        private string SendCommand = "Unknown";

        /// <summary>
        /// 「IDN」ボタンを押下するとき発生するイベントです。<NR></NR>
        /// IDNコマンドをclient1に投げて、返信を処理します。<NR></NR>
        /// </summary>
        /// <param name="sender">送信元オブジェクトです</param>
        /// <param name="e">イベントデータです</param>
        private void button1_Click(object sender, EventArgs e)
        {
            //IDNボタンを利用不可にします。
            button1.Enabled = false;

            //受信バッファのデータを破棄します。
            serialPort1.DiscardInBuffer();

            //IDNコマンドの文字列を準備する
            string sentTxt = "\u0002"+"IDN\r\n";
            byte[] sentMsg = Encoding.ASCII.GetBytes(sentTxt.ToCharArray());

            //シリアルポートにリモートコマンドを送信します。     
            serialPort1.Write(sentMsg, 0, sentMsg.Length);

            #region textBox1へ表示します
            textBox1.Text += String.Format("{0}\t{1}\t←{2}コマンドを送信しました。\r\n",
                    DateTime.Now.ToLongTimeString(),
                    serialPort1.PortName,
                    sentTxt.Substring(1, 3));
            #endregion

            do
            {
                //IDNコマンドへの返信があった場合の処理です。　　
                if (SendCommand.Equals("IDN"))
                {
                    //SendCommandを開放します。
                    SendCommand = "Unknown";

                    #region textBox1へ表示します
                    textBox1.Text += String.Format("{0}\t{1}\t→ 接続されているコントローラは{2}です。\r\n",
                            DateTime.Now.ToLongTimeString(),
                            serialPort1.PortName,
                            ControllerType);
                    #endregion

                    //ControllerTypeを開放します。
                    ControllerType = "Unknown";
                    break;
                }
            }
            while (true);

            //IDNボタンの利用をできるようにします。
            button1.Enabled = true;
        }

        /// <summary>
        /// serialPort1にデータを受信すると発生します
        /// </summary>
        /// <param name="sender">送信元オブジェクトです</param>
        /// <param name="e">イベントデータです</param>
        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            //受信バッファ内のバイト数を取得します。
            int bytes = this.serialPort1.BytesToRead;

            byte[] ReceivedBytes = new byte[bytes];
            char[] rc1 = new char[bytes];

            string ReceivedText = "";

            string[] Answers = new string[0];

            do
            {
                //受信バッファ内のバイトを読み取ります。
                serialPort1.Read(ReceivedBytes, 0, bytes);                

                for (int i = 0; i < ReceivedBytes.Length; i++)
                {
                    //読み取ったバイトをUTF-8に変換します。
                    ReceivedText += Convert.ToChar(ReceivedBytes[i]);

                    if (ReceivedBytes[i] == '\n')
                    {
                        //\rであればAnswers配列に格納します。
                        Answers = ReceivedText.Split('\t', '\r');

                        SendCommand = Answers[1];
                        ControllerType = Answers[2];
                        return;
                    }                   
                }
            } while (true);
        }
    }
}
