﻿using System;
using System.Text;
using System.Windows.Forms;

namespace KosmosSample
{
    /// <summary>
    /// このプログラムは神津精機製のコントローラARIESをTCP/IP通信によって制御するサンプルコード(C#)です。
    /// <NR></NR>
    /// このサンプルコードはARIESのTCP/IP制御のプログラムを作成する参考に作成されたものです。
    /// 商用・非商用を問わず実用を考慮した設計にはなっていません。
    /// <NR></NR>
    /// 弊社の環境にて動作確認をしておりますが、いかなる環境でも動作することを保証しておりません。
    /// <NR></NR>
    /// 起動をするとシリアルポートコンポーネントを立ち上げ、ポートを開きます。
    /// 次に「IDN」ボタンを押下するとIDNコマンドを送信し、型式を受信します。
    /// このアプリを閉じるときにポートを閉じます。
    /// <NR></NR>   
    /// このサンプルコードの著作権は神津精機株式会社に帰属します。
    /// Copylight　2018　Kohzu Precision Co., Ltd.
    /// written by Masato Okui
    /// </summary>
    public partial class Form1 : Form
    {
        /// <summary>
        /// メインフォームのインスタンスです。
        /// </summary>
        public Form1()
        {
            InitializeComponent();

            //TCP/IPポートを開きます。
            tcpIPClient1 = new System.Net.Sockets.TcpClient(IP_Address, Port);

            //データの送受信に利用するネットワークストリームを準備します。
            networkStream1 = tcpIPClient1.GetStream();

            #region textBox1へ表示します。
            textBox1.Text = String.Format("{0}\t{1}\tPort{2}に接続しました。\r\n",
                DateTime.Now.ToLongTimeString(), IP_Address, Port);
            #endregion
        }

        /// <summary>
        /// このフォームを閉じる直前に実行されます
        /// </summary>
        /// <param name="sender">送信元のオブジェクトです</param>
        /// <param name="e">イベントデータです</param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            //ネットワークストリームを閉じます。
            networkStream1.Close();

            //TCP/IPポートを閉じます。
            tcpIPClient1.Close();
        }

        /// <summary>
        /// TCPネットワーク接続用のクライアントです。
        /// </summary>
        private System.Net.Sockets.TcpClient tcpIPClient1;

        /// <summary>
        /// ARIESのTCP/IPアドレスです。
        /// </summary>
        private string IP_Address
        {
            get => "192.168.100.20";
        }

        /// <summary>
        /// ARIESのポート番号です。
        /// </summary>
        private int Port
        {
            get => 12321;
        }

        //データの送受信に利用するネットワークストリームです。
        private System.Net.Sockets.NetworkStream networkStream1;

        /// <summary>
        /// 「IDN」ボタンを押下するとき発生するイベントです。<NR></NR>
        /// IDNコマンドをclient1に投げて、返信を処理します。<NR></NR>
        /// </summary>
        /// <param name="sender">送信元オブジェクトです</param>
        /// <param name="e">イベントデータです</param>
        private void button1_Click(object sender, EventArgs e)
        {
            //IDNボタンを利用不可にします。
            button1.Enabled = false;

            //コントローラの型式を格納します。
            string controllerType = "Unknown";

            //送信コマンドを格納します。
            string receivedCommand = "Unknown";            

            //IDNコマンドの文字列を準備する
            string sentTxt = "IDN\r\n";
            byte[] sentBytes = Encoding.ASCII.GetBytes(sentTxt.ToCharArray());            

            //ネットワークストリームにリモートコマンドを送信します。     
            networkStream1.Write(sentBytes, 0, sentBytes.Length);

            //textBox1へ表示します
            textBox1.Text += String.Format("{0}\t{1}\t←{2}コマンドを送信しました。\r\n",
                    DateTime.Now.ToLongTimeString(),
                    IP_Address,
                    sentTxt.Substring(0, 3));

            //メモリを使用するストリームです。
            System.IO.MemoryStream memoryStream1 = new System.IO.MemoryStream();            

            byte[] receivedBytes = new byte[256];
            int receivedSize = 0;

            do
            {
                //neworkStreasm1のバッファから受信した文字列を読み取ります。
                receivedSize = networkStream1.Read(receivedBytes, 0, receivedBytes.Length);

                //文字を1文字以上受信したらmemoryStream1へ書き込みます。
                if (receivedSize > 0)
                {
                    memoryStream1.Write(receivedBytes, 0, receivedSize);
                }

                if (receivedBytes[receivedSize - 1].Equals(10))
                {
                    //memoryStrem1のバッファから文字列を読み取り、UTF-8にエンコーディングします。
                    string receivedText = System.Text.Encoding.UTF8.GetString(memoryStream1.GetBuffer(), 0, (int)memoryStream1.Length);

                    //memoryStream1を閉じます。
                    memoryStream1.Close();
                    

                    //デリミタのCRLFを除去します。
                    receivedText = receivedText.Trim('\r', '\n');

                    //受信した文字列を配列にします。
                    string[] Answers = receivedText.Split('\t');

                    receivedCommand = Answers[1];
                    controllerType = Answers[2];

                    break;
                }
            }
            while (networkStream1.DataAvailable);

            //IDNコマンドへの返信があった場合の処理です。　　
            if (receivedCommand.Equals("IDN"))
            {
                //SendCommandを開放します。
                receivedCommand = "Unknown";

                //textBox1へ表示します
                textBox1.Text += String.Format("{0}\t{1}\t→ 接続されているコントローラは{2}です。\r\n",
                        DateTime.Now.ToLongTimeString(),
                        IP_Address,
                        controllerType);

                //ControllerTypeを開放します。
                controllerType = "Unknown";
            }

            //IDNボタンの利用をできるようにします。
            button1.Enabled = true;
        }

    }
}
