﻿''' <summary>
''' このプログラムは神津精機製のコントローラARIESをTCP/ IP通信によって制御するサンプルコード(C#)です。
''' <NR></NR>
''' このサンプルコードはARIESのTCP/IP制御のプログラムを作成する参考に作成されたものです。
''' 商用・非商用を問わず実用を考慮した設計にはなっていません。
''' <NR></NR>
''' 弊社の環境にて動作確認をしておりますが、いかなる環境でも動作することを保証しておりません。
''' <NR></NR>
''' 起動をするとシリアルポートコンポーネントを立ち上げ、ポートを開きます。
''' 次に「IDN」ボタンを押下するとIDNコマンドを送信し、型式を受信します。
''' このアプリを閉じるときにポートを閉じます。
''' <NR></NR>   
''' このサンプルコードの著作権は神津精機株式会社に帰属します。
''' Copylight　2018　Kohzu Precision Co., Ltd.
''' written by Masato Okui
''' </summary>
Public Class Form1
    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load

        'TCP/IPポートを開きます。
        tcpIPClient1 = New System.Net.Sockets.TcpClient(IP_Address, Port)

        'データの送受信に利用するネットワークストリームを準備します。
        netWorkStream1 = tcpIPClient1.GetStream()

        'textBox1へ表示します。
        textBox1.Text = String.Format(DateTime.Now.ToLongTimeString() & vbTab & IP_Address & vbTab & "Port" & Port & "に接続しました。" & vbCrLf)

    End Sub

    ''' <summary>
    ''' このフォームを閉じる直前に実行されます
    ''' </summary>
    ''' <param name="sender">送信元オブジェクトです。</param>
    ''' <param name="e">イベントデータです。</param>
    Private Sub Form1_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
        'ネットワークストリームを閉じます。
        netWorkStream1.Close()

        'TCP/IPポートを閉じます。
        tcpIPClient1.Close()
    End Sub

    ''' <summary>   
    ''' TCPネットワーク接続用のクライアントです。
    '''</summary>
    Private tcpIPClient1 As System.Net.Sockets.TcpClient


    '''<summary>
    ''' ARIESのTCP/IPアドレスです。
    '''</summary>
    Private ReadOnly Property IP_Address As String
        Get
            Return "192.168.100.20"
        End Get
    End Property

    ''' <summary>
    ''' ARIESのポート番号です。
    ''' </summary>
    ''' <returns></returns>
    Private ReadOnly Property Port As Integer
        Get
            Return 12321
        End Get
    End Property

    ''' <summary>
    ''' データの送受信に利用するネットワークストリームです。
    ''' </summary>
    Private netWorkStream1 As System.Net.Sockets.NetworkStream

    Private Sub button1_Click(sender As Object, e As EventArgs) Handles button1.Click
        'IDNボタンを利用不可にします。
        button1.Enabled = False

        'コントローラの型式を格納します。
        Dim controllerType As String = "Unknown"

        '送信コマンドを格納します。
        Dim receivedCommand As String = "Unknown"

        'IDNコマンドの文字列を準備する
        Dim sentTxt As String = "IDN" & vbCrLf
        Dim sentBytes As Byte() = System.Text.Encoding.ASCII.GetBytes(sentTxt.ToCharArray())

        'ネットワークストリームにリモートコマンドを送信します。     
        netWorkStream1.Write(sentBytes, 0, sentBytes.Length)

        'textBox1へ表示します
        textBox1.Text +=
        String.Format(DateTime.Now.ToLongTimeString() & vbTab & IP_Address & vbTab &
        "←{0}コマンドを送信しました。" & vbCrLf, sentTxt.Substring(0, 3))

        'メモリを使用するストリームです。
        Dim memoryStream1 As System.IO.MemoryStream = New System.IO.MemoryStream()

        Dim receivedBytes As Byte() = New Byte(256) {}
        Dim receivedSize As Integer = 0

        Do
            'neworkStreasm1のバッファから受信した文字列を読み取ります。
            receivedSize = netWorkStream1.Read(receivedBytes, 0, receivedBytes.Length)

            '文字を1文字以上受信したらmemoryStream1へ書き込みます。
            If (receivedSize > 0) Then
                memoryStream1.Write(receivedBytes, 0, receivedSize)
            End If

            If receivedBytes(receivedSize - 1) = 10 Then

                'memoryStrem1のバッファから文字列を読み取り、UTF-8にエンコーディングします。
                Dim receivedText As String = System.Text.Encoding.UTF8.GetString(memoryStream1.GetBuffer(), 0, memoryStream1.Length)

                'memoryStream1を閉じます。
                memoryStream1.Close()

                'デリミタのCRLFを除去します。
                receivedText = receivedText.Trim(vbCrLf)

                '受信した文字列を配列にします。
                Dim Answers As String() = receivedText.Split(vbTab)

                receivedCommand = Answers(1)
                controllerType = Answers(2)

                Exit Do

            End If

        Loop 'While networkStream1.DataAvailable

        'IDNコマンドへの返信があった場合の処理です。　　
        If (receivedCommand.Equals("IDN")) Then

            'SendCommandを開放します。
            receivedCommand = "Unknown"

            'textBox1へ表示します
            textBox1.Text += String.Format(DateTime.Now.ToLongTimeString() &
                                           vbTab & IP_Address & vbTab &
                                           "→接続されているコントローラは{0}です。" &
                                           vbCrLf,
                                           controllerType)

            'ControllerTypeを開放します。
            controllerType = "Unknown"

        End If

        'IDNボタンの利用をできるようにします。
        button1.Enabled = True

    End Sub
End Class
